/*  Program Pyramida, version 0.3 (16. 9. 2002)
 *  Copyright (C) 2002  Martin Beranek <marble@matfyz.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "ai.h"

struct t_moves {
	int cnt;
	struct t_move *moves;
	int alloc;
};
#define MOVES_ALLOC_STEP	20

struct t_move *new_move(struct t_moves *m)
{
	m->cnt++;
	if (m->cnt > m->alloc) {
		m->alloc += MOVES_ALLOC_STEP;
		m->moves = realloc(m->moves, sizeof(struct t_move)*(m->alloc));
	}
	return &(m->moves[m->cnt-1]);
}

void free_moves(struct t_moves *m)
{
	m->cnt = 0;
	m->alloc = 0;
	if ((m->moves) != NULL)
		free(m->moves);
	m->moves = NULL;
}

#define CMP_RTP(x)	((struct t_move *)(x))
int move_cmp(const void *a, const void *b)
{
	return ((((CMP_RTP(b)->s1)>=0)?1:0) + (((CMP_RTP(b)->s31)>=0)?1:0) + (((CMP_RTP(b)->s32)>=0)?1:0)) -
		((((CMP_RTP(a)->s1)>=0)?1:0) + (((CMP_RTP(a)->s31)>=0)?1:0) + (((CMP_RTP(a)->s32)>=0)?1:0));
}

int move_cmp_mix(const void *a, const void *b)
{
	int r;
	r = ((((CMP_RTP(b)->s1)>=0)?1:0) + (((CMP_RTP(b)->s31)>=0)?1:0) + (((CMP_RTP(b)->s32)>=0)?1:0)) -
		((((CMP_RTP(a)->s1)>=0)?1:0) + (((CMP_RTP(a)->s31)>=0)?1:0) + (((CMP_RTP(a)->s32)>=0)?1:0));
	if (!r)
#ifdef __HAS_RANDOM__
		return (random()&0x1)?1:-1;
#else
		return (rand()&0x1)?1:-1;	// tohle u rand() moc nahodny neni, ale budiz... :-)
#endif
	else
		return r;
}
	

void generate_moves(struct t_game *gm, struct t_moves *mvs, int mix)
{
	struct t_move *m;
	int s1, s2, s31, s32;
	struct t_stone *st;

	st = gm->st;
	free_moves(mvs);

	for (s1=-1; s1<STONES; s1++) {
		recalc_board_m(gm);
		if (s1>=0) {
			if (st[s1].move != GET)
				continue;
			st[s1].state = NONE;
		}
		for (s2=0; s2<STONES; s2++) {
			recalc_board_m(gm);
			if (st[s2].move != ADD)
				continue;
			if ((s1 >= 0) && (st[s2].y >= st[s1].y))
				continue;
			m = new_move(mvs);
			m->s1 = s1;
			m->s2 = s2;
			m->s31 = -1; m->s32 = -1;
			st[s2].state = gm->p;
			if (check_sq(st, s2)) {
				for (s31=-1; s31<STONES; s31++) {
					recalc_board_m(gm);
					if (s31 >= 0) {
						if (st[s31].move != GET)
							continue;
						st[s31].state = NONE;
					}
					for (s32=s31+1; s32<STONES; s32++) {
						recalc_board_m(gm);
						if (st[s32].move != GET)
							continue;
						m = new_move(mvs);
						m->s1 = s1;
						m->s2 = s2;
						m->s31 = s31; m->s32 = s32;
					}
					if (s31 >= 0)
						st[s31].state = gm->p;
				}
			}
			st[s2].state = NONE;
		}
		if (s1>=0)
			st[s1].state = gm->p;
	}
	
	recalc_board_m(gm);

	if (mix) 
		qsort(mvs->moves, mvs->cnt, sizeof(struct t_move), move_cmp_mix);
	else
		qsort(mvs->moves, mvs->cnt, sizeof(struct t_move), move_cmp);
}

/* vraci hodnotu -AI_MAXVAL az AI_MAXVAL */
int st_value(struct t_game *gm)
{
	if (gm->p == WHITE)
		return (((gm->stones[WHITE])<<6) - ((gm->stones[BLACK])<<6) - (1<<5));
	else
		return (((gm->stones[BLACK])<<6) - ((gm->stones[WHITE])<<6) - (1<<5));
}

/* vrati hodnotu pozice (z poledu toho, kdo je podle gm->p na tahu) */
int pos_value(struct t_game *gm, int iter, int alpha, int beta)
{
	int np = (gm->p==WHITE)?BLACK:WHITE;
	int maxval, aval, i;
	struct t_moves m;

	m.moves = NULL;
	m.cnt = 0;
	m.alloc = 0;
	
	if (iter<=0) {
		return st_value(gm);
	}
	if (gm->stones[gm->p] <= 0)
		return -AI_MAXVAL;
	if (gm->stones[np] <= 0)
		return AI_MAXVAL;
	
	generate_moves(gm, &m, 0);

	maxval = alpha;
	for (i=0; i<m.cnt; i++) {
		do_move(gm, &(m.moves[i]), NULL, NULL);
		aval = -pos_value(gm, iter-1, -beta, -maxval);
		undo_move(gm, &(m.moves[i]));
		if (aval>maxval) {
			maxval = aval;
			if (aval>=beta) {
				free_moves(&m);
				return maxval;
			}
		}
	}

	free_moves(&m);
	return maxval;
}
	
int best_move(struct t_game *gm, struct t_move *bm_res)
{
	int maxval, aval, i, it;
	struct t_moves m;
	double x;

	m.moves = NULL;
	m.cnt = 0;
	m.alloc = 0;
	
	generate_moves(gm, &m, 1);

	/* tohle je dost problematicky - odhad vhodny hloubky prohledavani... */

	x = (double)(STONES - gm->stones[WHITE] - gm->stones[BLACK]);
	it = (int)((x/30.0)*(x/30.0)*(x/30.0)*(x/30.0)*(x/30.0)*10 + 3.5);
	
	if (m.cnt > 16) it--;
	
	maxval = -(AI_MAXVAL*2);
	for (i=0; i<m.cnt; i++) {
		do_move(gm, &(m.moves[i]), NULL, NULL);
		aval = -pos_value(gm, it, -(AI_MAXVAL*2), -maxval);
		if (aval>maxval) {
			copy_move(&(m.moves[i]), bm_res);
			maxval = aval;
		}

		undo_move(gm, &(m.moves[i]));
	}
	free_moves(&m);

	return maxval;
}


