/*  Program Pyramida, version 0.3 (16. 9. 2002)
 *  Copyright (C) 2002  Martin Beranek <marble@matfyz.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "game.h"


void init_stones(struct t_stone *st)
{
	int i, j;

	for (i=0; i<STONES; i++) {
		st[i].state = NONE;
		st[i].move = NONE;

		st[i].id = i;
		st[i].x = ID2B_X(i);
		st[i].y = ID2B_Y(i);
		st[i].z = ID2B_Z(i);
		st[i].above[0] = XYZ2ID(st[i].x-1, st[i].y-1, st[i].z-1);
		st[i].above[1] = XYZ2ID(st[i].x-1, st[i].y-1, st[i].z+1);
		st[i].above[2] = XYZ2ID(st[i].x+1, st[i].y-1, st[i].z-1);
		st[i].above[3] = XYZ2ID(st[i].x+1, st[i].y-1, st[i].z+1);
		st[i].below[0] = XYZ2ID(st[i].x-1, st[i].y+1, st[i].z-1);
		st[i].below[1] = XYZ2ID(st[i].x-1, st[i].y+1, st[i].z+1);
		st[i].below[2] = XYZ2ID(st[i].x+1, st[i].y+1, st[i].z-1);
		st[i].below[3] = XYZ2ID(st[i].x+1, st[i].y+1, st[i].z+1);
		for (j=0; j<4; j++) {
			st[i].sq[j][0] = XYZ2ID(st[i].x+((j<2)?2:-2), st[i].y, st[i].z);
			st[i].sq[j][1] = XYZ2ID(st[i].x+((j<2)?2:-2), st[i].y, st[i].z+((j&0x1)?2:-2));
			st[i].sq[j][2] = XYZ2ID(st[i].x, st[i].y, st[i].z+((j&0x1)?2:-2));
		}
	}
}

void init_game(struct t_game *gm)
{
	gm->stones[WHITE] = STONES>>1;
	gm->stones[BLACK] = STONES>>1;
	gm->p = WHITE;
	init_stones(gm->st);
	recalc_board_m(gm);
}

void copy_stones(struct t_stone *src, struct t_stone *dst)
{
	memcpy(dst, src, sizeof(struct t_stone) * STONES);
}

void copy_game(struct t_game *src, struct t_game *dst)
{
	dst->stones[0] = src->stones[0];
	dst->stones[1] = src->stones[1];
	dst->p = src->p;
	copy_stones(src->st, dst->st);
}

void copy_move(struct t_move *src, struct t_move *dst)
{
	memcpy(dst, src, sizeof(struct t_move));
}


void recalc_board_m_id(struct t_game *gm, int id)
{
	struct t_stone *st = gm->st;
	int j;

	if (st[id].state == NONE) {
		st[id].move = ADD;
		for (j=0; j<4; j++)
			if (!(IS_FULL(st, st[id].below[j]))) {
				st[id].move = NONE;
				break;
			}
	} else if (st[id].state == gm->p) {
		st[id].move = GET;
		for (j=0; j<4; j++)
			if (!(IS_FREE(st, st[id].above[j]))) {
				st[id].move = NONE;
				break;
			}
	} else {
		st[id].move = NONE;
	}
}

void recalc_board_m(struct t_game *gm)
{
	int i;

	for (i=0; i<STONES; i++) 
		recalc_board_m_id(gm, i);
}

void recalc_board_m_pos(struct t_game *gm, int id)
{
	struct t_stone *st = gm->st;
	int i;
	
	recalc_board_m_id(gm, id);
	for (i=0; i<4; i++)
		recalc_board_m_id(gm, st[id].above[i]);
	for (i=0; i<4; i++)
		recalc_board_m_id(gm, st[id].below[i]);
}

int check_sq(struct t_stone *st, int id)
{
	int i, w;

	w = st[id].state;
	if (w == NONE) return 0;

	for (i=0; i<4; i++)
		if (IS_COL(st, st[id].sq[i][0], w) && IS_COL(st, st[id].sq[i][1], w) && IS_COL(st, st[id].sq[i][2], w))
			return 1;
	return 0;
}

/* na zacatku nastavit step na 0  a pak postupny volani tehle funkce animuje cely tah */
int animate_move(struct t_game *gm, struct t_move *mv, int *step)
{
	switch (*step) {
		case 0:	// zacatek tahu
			(*step)++;
			if (mv->s1 >= 0) {
				gm->st[mv->s1].state = NONE;
				return 1;
			} else {
				gm->stones[gm->p]--;
			}
		case 1:
			(*step)++;
			gm->st[mv->s2].state = gm->p;
			if ((mv->s31 < 0) && (mv->s32 < 0)) {
				gm->p = (gm->p==WHITE)?BLACK:WHITE;
				recalc_board_m(gm);
				return 0;
			} else {
				return 1;
			}
		case 2:
			(*step)++;
			if (mv->s31 >= 0) {
				gm->st[mv->s31].state = NONE;
				gm->stones[gm->p]++;
				return 1;
			}
		case 3:
			(*step)++;
			if (mv->s32 >= 0) {
				gm->st[mv->s32].state = NONE;
				gm->stones[gm->p]++;
				return 1;
			}
		case 4:
			gm->p = (gm->p==WHITE)?BLACK:WHITE;
			recalc_board_m(gm);
			return 0;
	}
	return 0;
}


int do_move(struct t_game *gm, struct t_move *mv, int *amove, int *buf)
{
	if (amove == NULL) {
		/* cely tah najednou */
		if (mv->s1 >= 0)
			gm->st[mv->s1].state = NONE;
		else
			gm->stones[gm->p]--;

		if (mv->s2 < 0) return MOVE_ERR;
		gm->st[mv->s2].state = gm->p;
		
		if (mv->s31 >= 0) {
			gm->st[mv->s31].state = NONE;
			gm->stones[gm->p]++;
		}
		if (mv->s32 >= 0) {
			gm->st[mv->s32].state = NONE;
			gm->stones[gm->p]++;
		}
		gm->p = (gm->p==WHITE)?BLACK:WHITE;
		recalc_board_m(gm);
		return MOVE_OK;
	} else {
		switch (*amove) {
			case AMOVE_GET0:
				if (mv->s1 < 0)
					return MOVE_ERR;
				gm->st[mv->s1].state = NONE;
				*amove = AMOVE_PUT;
				recalc_board_m(gm);
				return MOVE_OK;

			case AMOVE_PUT:
				if (mv->s2 < 0)
					return MOVE_ERR;
				gm->st[mv->s2].state = gm->p;
				if (mv->s1 == mv->s2) {
					mv->s1 = -1;
					*buf = 1;
					recalc_board_m(gm);
					return MOVE_OK;
				}
				if (*buf > 0) {
					*buf = 0;
					gm->stones[gm->p]--;
				}
				if (check_sq(gm->st, mv->s2)) {
					*amove = AMOVE_GET1;
					recalc_board_m(gm);
					return MOVE_OK;
				}
				*amove = AMOVE_END;
				gm->p = (gm->p==WHITE)?BLACK:WHITE;
				recalc_board_m(gm);
				return MOVE_OK;

			case AMOVE_GET1:
				if (mv->s31 < 0)
					return MOVE_ERR;
				gm->st[mv->s31].state = NONE;
				gm->stones[gm->p]++;
				*amove = AMOVE_GET2;
				recalc_board_m(gm);
				return MOVE_OK;
		
			case AMOVE_GET2:
				if (mv->s32 < 0)
					return MOVE_ERR;
				gm->st[mv->s32].state = NONE;
				gm->stones[gm->p]++;
				*amove = AMOVE_END;
				gm->p = (gm->p==WHITE)?BLACK:WHITE;
				recalc_board_m(gm);
				return MOVE_OK;
		}
		return MOVE_ERR;
	}
}

int rclick_move(struct t_game *gm, struct t_move *mv, int *amove, int *buf)
{
	if (amove == NULL) return MOVE_ERR;
	switch (*amove) {
		case AMOVE_GET0:
			*buf = 1;
			*amove = AMOVE_PUT;
			mv->s1 = -1;
			return MOVE_OK;

		case AMOVE_PUT:
			*amove = AMOVE_GET0;
			if (mv->s1 >= 0)
				gm->st[mv->s1].state = gm->p;
			*buf = 0;
			recalc_board_m(gm);
			return MOVE_OK;
				
		case AMOVE_GET1:
		case AMOVE_GET2:
			*amove = AMOVE_END;
			gm->p = (gm->p==WHITE)?BLACK:WHITE;
			return MOVE_OK;
	}
	return MOVE_ERR;
}

int undo_move(struct t_game *gm, struct t_move *mv)
{
	gm->p = (gm->p==WHITE)?BLACK:WHITE;
	if (mv->s31 >= 0) {
		gm->st[mv->s31].state = gm->p;
		gm->stones[gm->p]--;
	}
	if (mv->s32 >= 0) {
		gm->st[mv->s32].state = gm->p;
		gm->stones[gm->p]--;
	}
	if (mv->s2 < 0) return MOVE_ERR;
	gm->st[mv->s2].state = NONE;

	if (mv->s1 >= 0)
		gm->st[mv->s1].state = gm->p;
	else
		gm->stones[gm->p]++;

	recalc_board_m(gm);
	return MOVE_OK;
}


