#ifndef __GAME_H__
#define __GAME_H__

#include <malloc.h>
#include <string.h>

/*
 * 
 *  \|                     x
 *  -+--------------------->
 *   |\
 *   | \
 *   |  \
 *   |  _\| z (za obrazovku)
 *   |  
 *   |
 *   |
 *   v y
 *
 * 
 * pole board - rozestaveni kamenu;  0 - 15: spodni vrstva ... [-3R; 1Ry; -3R], [3R; 1Ry; 3R]
 * 				    16 - 24: druha vrstva  ... [-2R; 0; -2R], [2R; 0; 2R]
 * 				    25 - 28: treti vrstva  ... [-1R; -1Ry; -1R], [1R; -1Ry; 1R]
 * 				         29: horni vrstva  ... [0; -2Ry; 0]
 */


/* prevadi ID kulicky na souradnice na desce ([0,0,0] je druha vrstva odspod uprostred; vertikalne po jedne,
 * 	horizontalne po dvou) */
#define ID2B_X(id)	(((id)<16)?((id)&0x3)*2-3:((id)<25)?(((id)-16)%3)*2-2:((id)<29)?(((id)-25)%2)*2-1:0)
#define ID2B_Y(id)	(((id)<16)?1:((id)<25)?0:((id)<29)?-1:-2)
#define ID2B_Z(id)	(((id)<16)?(((id)&0xC)>>1)-3:((id)<25)?(((id)-16)/3)*2-2:((id)<29)?(((id)-25)&0x2)-1:0)

#define MABS(x)	((x)<0?-(x):(x))

/* prevadi souradnice na desce na ID kulicky (vraci -1 pro chybne souradnice) */
#define XYZ2ID(x,y,z)	( ( ((MABS(x)-(y))>2) || ((MABS(z)-(y))>2) )?-1: \
		((y)==1)?((((z)+3)<<1)+(((x)+3)>>1)): \
		((y)==0)?( ((((z)+2)>>1)*3)+(((x)+2)>>1)+16 ): \
		((y)==-1)?( (((z)+1)&0x2)+(((x)+1)>>1)+25): \
		((y)==-2)?29:-1)

#define STONES	30

#define NONE	-1
#define WHITE	0
#define BLACK	1

#define ADD	0
#define GET	1

struct t_stone {
	// menici se slozky
	int state;		// stav pozice - volna, nebo bily/cerny
	int move;		// mozny tah na teto pozici - pridat/odebrat
	
	// staticke slozky - pro urychleni vypoctu
	int id;			// pro zpetne urceni pri pouziti setrideneho pole
	int x, y, z;		// souradnice kamenu na desce
	int above[4];		// id kamenu nad nim - pokud jsou prazdne, tak se muze odebrat
	int below[4];		// id kamenu pod nim - pokud jsou plne, tak se muze polozit
	int sq[4][3];		// id kamenu ve ctvercich okolo - prvni index ctverec, druhy kamen v nem
};

/* struktura, ve ktere je ulozen stav hry - tj. kde je kolik kamenu a kdo hraje */
struct t_game {
	int stones[2];		// kameny v zasobe
	int p;			// aktualni hrac
	
	struct t_stone st[STONES];
};

/* struktura s informacema o hraci - melo by existovat pole se dvema polozkama */
struct t_player {
	char *name;
	int ai;		// hraje za nej comp? TODO: ruzny hodnoty >0  <=>  ruzny obtiznosti ???
};

/* zjisti, je-li pozice prazdna nebo ne; pokud je mimo desku, tak vrati true */
#define IS_FREE(brd, id)	((((id)>=0)&&((id)<STONES))?(brd[id].state)==NONE:1)
#define IS_FULL(brd, id)	((((id)>=0)&&((id)<STONES))?(brd[id].state)!=NONE:1)

/* zjisti, je-li pozice vyplnena danou barvou; pokud je mimo desku, tak vrati false */
#define IS_COL(brd, id, c)	((((id)>=0)&&((id)<STONES))?(brd[id].state)==(c):0)

/* vrati true, pokud je hra uz vyhrana - tj. jeden hrac ma nula kamenu */
#define IS_WON(gm)		((gm.stones[WHITE]==0)||(gm.stones[BLACK]==0))


#define MOVE_ERR	-1
#define MOVE_OK		 0


/* 's1' je id kamenu, ktery se sebere (-1 pokud se bere ze zasobniku);
 * 's2' je id pozice, kam se ten kamen poklada
 * 's31' a 's32' jsou pozice kamenu, ktere se na konci tahu berou (nebo -1 pokud se neberou) */
struct t_move {
	int s1, s2, s31, s32;
};

#define AMOVE_END	0	// konec tahu
#define AMOVE_GET0	1	// sbiram prvni kamen
#define AMOVE_PUT	2	// pokladam kamen
#define AMOVE_GET1	3	// sbiram prvni kamen po vytvoreni ctverce
#define AMOVE_GET2	4	// sbiram druhy kamen po vytvoreni ctverce


/* nastavi staticke slozky kamenu a vynuluje dynamicke */
void init_stones(struct t_stone *st);

/* inicializuje hru plus jeji kameny */
void init_game(struct t_game *gm);

/* zkopiruje slozky pole t_stone, prip. t_game nebo t_move */
void copy_stones(struct t_stone *src, struct t_stone *dst);
void copy_game(struct t_game *src, struct t_game *dst);
void copy_move(struct t_move *src, struct t_move *dst);

/* doplni aktualni mozne tahy ADD/GET */
void recalc_board_m(struct t_game *gm);

/* to same, ale pouze pokud se zmenil jen kamen na pozici 'id' */
void recalc_board_m_pos(struct t_game *gm, int id);

/* zkontroluje vytvoreni ctverce kamenem na miste 'id' */
int check_sq(struct t_stone *st, int id);

/* provede dany tah (bez kontroly jeho platnosti) */
/* pokud jsou nejake potrebne id nahrazeny -1, berou se tak, ze zatim nebyly urceny, nebo
 * ze uz byly provedeny drive */
int do_move(struct t_game *gm, struct t_move *mv, int *amove, int *buf);

int rclick_move(struct t_game *gm, struct t_move *mv, int *amove, int *buf);

/* na zacatku nastavit step na 0  a pak postupny volani tehle funkce animuje cely tah */
int animate_move(struct t_game *gm, struct t_move *mv, int *step);

/* vrati tah - tady ten tah musi byt kompletni! */
int undo_move(struct t_game *gm, struct t_move *mv);

#endif

