/*  Program Pyramida, version 0.3 (16. 9. 2002)
 *  Copyright (C) 2002  Martin Beranek <marble@matfyz.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



#include "gfx.h"

int init_game_gfx(struct t_game_gfx *gm_gfx, struct t_game *gm)
{
	int i;
	
	set_trans_blender(0, 0, 0, 128);

	/* pocatecni natoceni desky */
	gm_gfx->u1 = itofix(32);
	gm_gfx->u2 = itofix(24);

	for (i=0; i<STONES; i++)
		gm_gfx->s_st[i] = i;

	gm_gfx->ghost = NONE;

	gm_gfx->gm = gm;
	gm_gfx->st = gm->st;

	recalc_board_gfx(gm_gfx);

	if (gm_gfx->bg == NULL)
		return 0;
	else
		return 1;
}

/* pro qsort u recalc_board_gfx */
struct t_stone_gfx *qsort_st;

int cmp_dst (const void *a, const void *b)
{
	return - qsort_st[*((int *)a)].dst + qsort_st[*((int *)b)].dst;
}

void recalc_board_gfx(struct t_game_gfx *gm)
{
	MATRIX x_mat, y_mat;
	int i;
	fixed x, y, z, tx, ty, tz;

	get_x_rotate_matrix(&x_mat, gm->u1);
	get_y_rotate_matrix(&y_mat, gm->u2);
	matrix_mul(&y_mat, &x_mat, &(gm->m));

	for (i=0; i<STONES; i++) {
		x = itofix(gm->st[i].x * R);
		y = itofix(gm->st[i].y * Ry);
		z = itofix(gm->st[i].z * R);
		apply_matrix(&(gm->m), x, y, z, &tx, &ty, &tz);
		
		gm->g_st[i].dst = vector_length(tx-cam_x, ty-cam_y, tz-cam_z);

		/* 3D -> 2D:	x = (x * cam_foc) / (z - cam_z) */
		gm->g_st[i].gx = fixtoi(fixmul(tx, fixdiv(cam_foc, tz-cam_z)));
		gm->g_st[i].gy = fixtoi(fixmul(ty, fixdiv(cam_foc, tz-cam_z)));
		/*gm->g_st[i].gx = fixtoi(fixdiv(fixmul(tx,cam_foc), z-cam_z));
		gm->g_st[i].gy = fixtoi(fixdiv(fixmul(ty,cam_foc), z-cam_z));*/

		/* prumer kulicky - nepocita se "korektne" proto, aby vychazely spravne vedle sebe */
		gm->g_st[i].gd = fixtoi(fixmul(itofix(2*R), fixdiv(cam_foc, tz-cam_z)));
		/*gm->g_st[i].gd = fixtoi(fixdiv(fixmul(itofix(2*R),cam_foc),z-cam_z));*/
	}

	qsort_st = gm->g_st;
	qsort(gm->s_st, STONES, sizeof(int), cmp_dst);
}

/* nakresli kamen (*s) na bitmapu *to na pozici [x; y], ghost urcuje prusvitnost */
/* pozice je relativne ke stredu bitmapy */
void draw_stone(BITMAP *to, BITMAP *s, int x, int y, int ghost, int sz)
{
	int gx, gy;
	BITMAP *b_gh;
	
	gx = x + (BRDW_W>>1) - (sz>>1);
	gy = y + (BRDW_H>>1) - (sz>>1);
	
	if (ghost) {
		b_gh = create_bitmap(sz, sz);
		blit(to, b_gh, gx, gy, 0, 0, sz, sz);
		stretch_sprite(b_gh, s, 0, 0, sz, sz);
		draw_trans_sprite(to, b_gh, gx, gy);
		destroy_bitmap(b_gh);
	} else {
		stretch_sprite(to, s, gx, gy, sz, sz);
	}
}

void draw_stones(struct t_game_gfx *gm)
{
	int i, id;
	
	draw_sprite(gm->bg, gm->b_bg, 0, 0);

	for (i=0; i<STONES; i++) {
		id = gm->s_st[i];
		if (gm->ghost == id) {
			if (gm->st[id].y == 1)		/* je-li to spodni kamen */
				draw_stone(gm->bg, gm->b_none, gm->g_st[id].gx, gm->g_st[id].gy,
						0, (gm->g_st[id].gd)>>2);
			if (gm->st[id].state == NONE)
				draw_stone(gm->bg, gm->b_st[gm->gm->p], gm->g_st[id].gx, gm->g_st[id].gy,
						1, gm->g_st[id].gd);
			else
				draw_stone(gm->bg, gm->b_st[gm->st[id].state], gm->g_st[id].gx, gm->g_st[id].gy,
						1, gm->g_st[id].gd);
		} else {
			if (gm->st[id].state == NONE) {
				if (gm->st[id].y == 1)		/* je-li to spodni kamen */
					draw_stone(gm->bg, gm->b_none, gm->g_st[id].gx, gm->g_st[id].gy,
							0, (gm->g_st[id].gd)>>2);
			} else {
				draw_stone(gm->bg, gm->b_st[gm->st[id].state], gm->g_st[id].gx, gm->g_st[id].gy,
						0, gm->g_st[id].gd);
			}
		}
	}
}

void blit_board(struct t_game_gfx *gm, int x, int y)
{
	//draw_sprite(screen, gm->bg, x, y);
	blit(gm->bg, screen, 0, 0, x, y, gm->bg->w, gm->bg->h);
}



/* TODO: t_stone .. move !!!!! */

/* najde kamen na dane pozici ([0, 0] uprostred obrazovky) */
/* vraci -1 pokud krysa neukazuje na zadny kamen */
/* Parametr max_y urcuje nejvysssi y-ovou souradnici kamenu, ktere se pocitaji,
 * orig_id urcuje id kamenu, ktere se bere i pokud nesouhlasi vyska (pro navrat zpet) */
int get_stone_id(struct t_game_gfx *gm, int mx, int my, int add, int max_y, int orig_id)
{
	int i;
	long int dst;
	struct t_stone_gfx *sg;
	struct t_stone *st;

	for (i = STONES - 1; i >= 0; i--) {
		sg = &(gm->g_st[gm->s_st[i]]);
		st = &(gm->st[gm->s_st[i]]);
		dst = (mx-sg->gx)*(mx-sg->gx) + (my-sg->gy)*(my-sg->gy);
		if (dst <= ((sg->gd*sg->gd)/4)) {
			if ((add) && (st->move == ADD)) {
				if ((st->y <= max_y) || (st->id == orig_id))
					return st->id;
			} else if ((!add) && (st->move == GET)) {
				return st->id;
			} else {
				if (st->state != NONE)
					return -1;
			}
		}
	}
	return -1;
}

#define BUF_SZ	(R)
#define BUF_SPC	(BUF_SZ>>2)

void draw_bufs(struct t_game_gfx *gm, int gh, int x, int y, int p)
{
	BITMAP *bg, *b_gh;
	int w, h;
	int i;
	int gh_x, gh_y;

	w = gm->b_left->w;
	h = gm->b_left->h;
	bg = create_bitmap(w, h);

	if (p == WHITE) {
		blit(gm->b_left, bg, 0, 0, 0, 0, w, h);
		for (i=0; i<(gm->gm->stones[WHITE]-1); i++) {
			stretch_sprite(bg, gm->b_st[WHITE], (w>>1) - (BUF_SZ>>1) + ((i&0x1)?1:-1)*((BUF_SZ+BUF_SPC)>>1),
					h - BUF_SZ - BUF_SPC - (i>>1)*(BUF_SZ+BUF_SPC),	BUF_SZ, BUF_SZ);
		}
		if (gm->gm->stones[WHITE] > 0) {
			gh_x = (w>>1) - (BUF_SZ>>1) + ((i&0x1)?1:-1)*((BUF_SZ+BUF_SPC)>>1);
			gh_y = h - BUF_SZ - BUF_SPC - (i>>1)*(BUF_SZ+BUF_SPC);
			if (gh == WHITE) {
				b_gh = create_bitmap(BUF_SZ, BUF_SZ);
				blit(bg, b_gh, gh_x, gh_y, 0, 0, w, h);
				stretch_sprite(b_gh, gm->b_st[WHITE], 0, 0, BUF_SZ, BUF_SZ);
				draw_trans_sprite(bg, b_gh, gh_x, gh_y);
				destroy_bitmap(b_gh);
			} else {
				stretch_sprite(bg, gm->b_st[WHITE], gh_x, gh_y, BUF_SZ, BUF_SZ);
			}
		}
		blit(bg, screen, 0, 0, x, y, w, h);
	} else {
		blit(gm->b_right, bg, 0, 0, 0, 0, w, h);
		for (i=0; i<(gm->gm->stones[BLACK]-1); i++)
			stretch_sprite(bg, gm->b_st[BLACK], (w>>1) - (BUF_SZ>>1) + ((i&0x1)?-1:+1)*((BUF_SZ+BUF_SPC)>>1),
					h - BUF_SZ - BUF_SPC - (i>>1)*(BUF_SZ+BUF_SPC), BUF_SZ, BUF_SZ);
		if (gm->gm->stones[BLACK] > 0) {
			gh_x = (w>>1) - (BUF_SZ>>1) + ((i&0x1)?-1:1)*((BUF_SZ+BUF_SPC)>>1);
			gh_y = h - BUF_SZ - BUF_SPC - (i>>1)*(BUF_SZ+BUF_SPC);
			if (gh == BLACK) {
				b_gh = create_bitmap(BUF_SZ, BUF_SZ);
				blit(bg, b_gh, gh_x, gh_y, 0, 0, w, h);
				stretch_sprite(b_gh, gm->b_st[BLACK], 0, 0, BUF_SZ, BUF_SZ);
				draw_trans_sprite(bg, b_gh, gh_x, gh_y);
				destroy_bitmap(b_gh);
			} else {
				stretch_sprite(bg, gm->b_st[BLACK], gh_x, gh_y, BUF_SZ, BUF_SZ);
			}
		}
		blit(bg, screen, 0, 0, x, y, w, h);
	}

	destroy_bitmap(bg);
}


