/*  Program Pyramida, version 0.3 (16. 9. 2002)
 *  Copyright (C) 2002  Martin Beranek <marble@matfyz.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



#include <stdlib.h>
#include <time.h>

#include <allegro.h>
#include "pyr.h"
#include "game.h"
#include "data.h"
#include "ai.h"
#include "ui.h"
#include "gfx.h"

BITMAP *bg;
struct t_game game;
struct t_game_gfx game_gfx;
struct t_player players[2];


int main(int argc, char *argv[])
{
	int i, bpp, gfx_m;

#ifdef __HAS_RANDOM__
	srandom(time(NULL));
#else
	srand(time(NULL));
#endif
	
	set_uformat(U_UTF8);
	
	allegro_init();

	install_keyboard(); 
	install_timer(); 
	install_mouse(); 
	
	lock_timer_stuff();	

	if (get_config_int(CFG_SECTION_NM, "windowed", 0))		// ma se to spustit v okne?
		gfx_m = GFX_AUTODETECT_WINDOWED;
	else 
		gfx_m = GFX_AUTODETECT_FULLSCREEN;
	
	bpp = get_config_int(CFG_SECTION_NM, "bpp", 0);		// nastaveni barevny hloubky...
	if (!((bpp == 15) || (bpp == 16) || (bpp == 24) || (bpp == 32) || (bpp == 0))) {
		set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
		allegro_message("Invalid color depth %i specified in config file.\n", bpp);
		return -1;
	}
	if (!bpp) {
		/* no fuj.... ;-)) Ale otesti to vscehny pouzitelny barevny hloubky...
		 * ...lip to neumim (a Allegro asi taky ne). */
		set_color_depth(32);
		if (set_gfx_mode(gfx_m, 800, 600, 0, 0) != 0) {
			set_color_depth(24);
			if (set_gfx_mode(gfx_m, 800, 600, 0, 0) != 0) {
				set_color_depth(16);
				if (set_gfx_mode(gfx_m, 800, 600, 0, 0) != 0) {
					set_color_depth(15);
					if (set_gfx_mode(gfx_m, 800, 600, 0, 0) != 0) {
						set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
						allegro_message("Unable to set graphic mode 800x600\n%s\n", allegro_error);
						return -1;
					}
				}
			}
		}
	} else {
		set_color_depth(bpp);
		if (set_gfx_mode(gfx_m, 800, 600, 0, 0) != 0) {
			set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
			allegro_message("Unable to set graphic mode 800x600\n%s\n", allegro_error);
			return -1;
		}
	}

	game_gfx.bg = create_bitmap(BRDW_W, BRDW_H);
	
	init_game(&game);
	init_game_gfx(&game_gfx, &game);

	if (!load_images(&game_gfx, &bg)) {
		set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
		allegro_message("Cannot load game data!\n");
		return -1;
	}
	

	/* nacte udaje o hracich z configfile */
	players[WHITE].name = ustrdup(get_config_string(CFG_SECTION_NM, "nm1", "Bily"));
	players[BLACK].name = ustrdup(get_config_string(CFG_SECTION_NM, "nm2", "Cerny"));
	players[WHITE].ai = get_config_int(CFG_SECTION_NM, "ai1", 0);
	players[BLACK].ai = get_config_int(CFG_SECTION_NM, "ai2", 1);

	init_main_d(&game_gfx, players);
	
	draw_sprite(screen, bg, 0, 0);
	dialog_message(main_d, MSG_START, 0, &i);
	dialog_message(main_d, MSG_DRAW, 0, &i);
	dialog_message(main_d, MSG_END, 0, &i);

	do {
		if (!do_menu_dialog(players))
			break;
		init_game(&game);
		init_game_gfx(&game_gfx, &game);
		main_d_exit = 0;
		do_dialog(main_d, 1);
	} while (main_d_exit != 1);
	
	set_config_string(CFG_SECTION_NM, "nm1", players[WHITE].name);
	set_config_string(CFG_SECTION_NM, "nm2", players[BLACK].name);
	set_config_int(CFG_SECTION_NM, "ai1", players[WHITE].ai);
	set_config_int(CFG_SECTION_NM, "ai2", players[BLACK].ai);
	
	destroy_bitmap(game_gfx.bg);

	set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
	release_data();
	return 0;
}


END_OF_MAIN();

